/* CreateComponentInfo.java - hades.manager.CreateComponentInfo
 *
 * an utility class to construct a ComponentInfo (.clb) file
 * for a given HADES simobject
 *
 * 12.03.01 - first version
 */
package hades.manager;

import java.io.*;
import java.awt.*;

import hades.gui.*;
import hades.models.Design;
import jfig.utils.SetupManager;

/**
 * a class to generate .clb ComponentInfo files for Hades SimObjects.
 */
public class CreateComponentInfo {

  private String author  = "-";
  private String version = "1.001";
  private String description = "";


  public CreateComponentInfo() {
  }


  public void createComponentInfo( String filename ) throws Exception {
    String classname = filename.
                           substring( 0, filename.lastIndexOf( '.' )).
                             replace( '/', '.' );
    String imagepath = filename.
                          substring( 0, filename.lastIndexOf( '.' )) + ".gif";
    String sourcepath = filename.
                          substring( 0, filename.lastIndexOf( '.' )) + ".java";
    String symbolpath = filename.
                          substring( 0, filename.lastIndexOf( '.' )) + ".sym";

    String clbname =  filename.
                          substring( 0, filename.lastIndexOf( '.' )) + ".clb";

    try {
      Object o = Class.forName( classname ).newInstance();
      if (!(o instanceof hades.simulator.SimObject)) return;
    }
    catch( Exception e ) {
      msg( "-E- could not instantiate class: " + classname );
      return;
    }

    ComponentInfo clb = new ComponentInfo();

    clb.addFile( filename );
    clb.addFile( sourcepath );
    clb.addFile( symbolpath );
    clb.addFile( imagepath );

    clb.setStart( classname );
    clb.setPathname( clbname );
    clb.setAuthor( author );
    clb.setVersion( version );
    clb.setName( classname );
    clb.setImagepath( imagepath );
    clb.setIconpath( "simobj.gif" );
    clb.setType( "simobject" );
    clb.setDescription( description );
    clb.saveComponentInfo();
  }






  public static void usage() {
    msg( "Usage: java hades.manager.CreateComponentInfo [options] [files]\n"
    + "where options include:\n"
    + "-author name        default: -\n"
    + "-version name       default: 1.001\n"
    + "-description filename  default: none\n"
    + "files is a list of .class files\n"
    + "and the resulting files will be written to <classname>.clb\n"
    + "Example(s):\n"
    + "java hades.manager.CreateComponentInfo models/gates/Inv.class \n"
    + "java hades.manager.CreateComponentInfo -author FNH -version 42 -description /tmp/foo.txt hades/models/gates/And2.class \n"
    );
  }


  public static void msg( String s ) {
    System.out.println( s );
  }


  public static void main( String argv[] ) {
    try {
      CreateComponentInfo cci = new CreateComponentInfo();
      cci.description = "XXX\n";

      int i = 0;
      if ("-author" .equals( argv[i] )) {
         cci.author = argv[i+1];
         i += 2;
      } 
      if ("-version".equals( argv[i] )) {
         cci.version = argv[i+1];
         i += 2;
      }
      if ("-description".equals( argv[i] )) {
         cci.description = 
           DesignManager.getDesignManager().
             readStreamIntoBuffer( argv[i+1] ).toString();
         i += 2;
      }
     
      //jfig.utils.ExceptionTracer.setEnabled( false );
      //SetupManager.loadGlobalProperties( "hades/.hadesrc" );
      //SetupManager.loadUserProperties ( ".hadesrc" );
      //
      //Editor  editor = new Editor();

      for( ; i < argv.length; i++ ) {
        try {
          cci.createComponentInfo( argv[i] );
        }
        catch( Exception e ) {
          msg( "-W- failed for " + argv[i] );
          msg( "-W- exception is: " + e );
        }
      }
      System.exit( 0 );
    }
    catch( Exception e ) {
      usage();
    }
  }
}
