/* HadesTreeCellRenderer - hades.manager.HadesTreeCellRenderer
 *
 * a CellRenderer responsible for the Colibri tree appearance
 *
 * 28.02.01 - some more icon types
 * 27.02.01 - update to Swing 1.1, integrate to Hades 0.87, FNH
 * 01.07.98 - original version by
 * @author    Ulrich Dallmann       (2dallman@informatik.uni-hamburg.de)
 * @author    Karola Kroenert       (2kroene@informatik.uni-hamburg.de
 */
package hades.manager;

import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;
import hades.manager.tree.*;


public class  HadesTreeCellRenderer 
       extends  DefaultTreeCellRenderer 
{

  public final static String iconPath = "/hades/manager/icons/";

  /** 
   * icons for tree root: default, system (classpath), local (filesystem),
   * zip (jar), server, search result, collapsed and expanded directories,
   * basic content types.
   */
  static ImageIcon          
                     defaultIcon,
                     serverIcon,
                     systemIcon,
                     localIcon,
                     jarzipIcon,
                     searchIcon,

                     expandedIcon,
                     collapsedIcon,

                     simobjIcon, 
                     designIcon, 
                     memoryIcon,
                     textIcon,
                     pythonIcon;


  /** 
   * Color to use for the background when selected. 
   */
  static protected final Color SelectedBackgroundColor = new Color(0, 0, 128);
 
  static {
    loadDefaultIcons();
  } 

  private static void loadDefaultIcons() {
    //System.out.println( "icons..." );
    defaultIcon     = loadIcon( iconPath, "default.gif" );

    expandedIcon    = loadIcon( iconPath, "expanded.gif" );
    collapsedIcon   = loadIcon( iconPath, "collapsed.gif" );

    systemIcon      = loadIcon( iconPath, "system.gif" );
    localIcon       = loadIcon( iconPath, "local.gif" );
    serverIcon      = loadIcon( iconPath, "server.gif" );
    searchIcon      = loadIcon( iconPath, "search.gif" );
    jarzipIcon      = loadIcon( iconPath, "jarzip.gif" );

    simobjIcon      = loadIcon( iconPath, "simobj.gif" );
    designIcon      = loadIcon( iconPath, "design.gif" );
    pythonIcon      = loadIcon( iconPath, "python.gif" );
    textIcon        = loadIcon( iconPath, "text.gif" );
    memoryIcon      = loadIcon( iconPath, "memory.gif" );
    // XXXIcon      = loadIcon( iconPath, "XXX" );
    //System.out.println( "icons ok." );
  }


  /**
   * utility method to load an ImageIcon from a system resource.
   * In an ideal world, this method would be provided by Swing :-(
   */
  private static ImageIcon loadIcon( String path, String name ) {
    String resourcename = path + name;
    try {
      return new ImageIcon( 
        ImageIcon.class.getResource( resourcename ));
        //ClassLoader.getSystemResource( resourcename );
    }
    catch( Exception e ) {
      System.out.println( "-E- " + e + " " + resourcename );
      return null;
    }
  }



  /**
   * create a TreeCellRenderer useful for the Colibri object tree.
   */
  public HadesTreeCellRenderer() {
    super();
  }





  public Component getTreeCellRendererComponent(
                                 JTree tree,
                                 Object value,
                                 boolean sel,
                                 boolean expanded,
                                 boolean leaf,
                                 int row,
                                 boolean hasFocus) 
  {
     super.getTreeCellRendererComponent(
                                 tree, value, sel,
                                 expanded, leaf, row,
                                 hasFocus);

     //Object tmp = null;
     //try {
     //  tmp = ((DefaultMutableTreeNode) value).getUserObject();
     //}
     //catch( Exception e ) {
     //System.out.println( "-#- internal: cast failed: " + tmp );
     //}
     //System.out.println( value.getClass() + " " + value );

     SortedTreeNode node = null;
     try {
       node = (SortedTreeNode) value;
     }
     catch( Exception e ) {}
     if (node == null) {
       System.out.println( "-#- internal: cast failed: " + value );
       setIcon( defaultIcon );
     }
     else {
       ImageIcon icon = node.getIcon();
       if (icon == null) icon = textIcon;
       setIcon( icon );
     }

/*
     if (leaf ) {
       String s = value.toString();
       if (s.endsWith( ".hds" ))      setIcon( designIcon );
       else if (s.endsWith( ".rom" )) setIcon( textIcon );
       else if (s.endsWith( ".sym" )) setIcon( simobjIcon );
       else if (s.endsWith( ".jar" )) setIcon( jarzipIcon );
       else if (s.endsWith( ".zip" )) setIcon( jarzipIcon );
       else if (s.endsWith( ".py" ))  setIcon( pythonIcon );
       else if (s.endsWith( ".pyc" )) setIcon( pythonIcon );
       else                           setIcon( defaultIcon );

       //if (is_1(value)) setIcon( simobjIcon );
       //else             setIcon( designIcon );
     }
     else {
       String s = value.toString();
       if (s.endsWith( ".zip" ) || s.endsWith( ".jar" )) {
         setIcon( jarzipIcon );
       }
       else if ( expanded ) {
         setIcon( expandedIcon );
       }
       else {
         setIcon( collapsedIcon );
       }
     }
*/

     return this;
   }


   

    public boolean is_1( Object value ) {
      return value.toString().indexOf( "1" ) > 0;
    }


  
  /** Whether or not the item that was last configured is selected. */
  protected boolean       selected;
  protected boolean       hasFocus;
  
  
}
