/* InfoPanel.java - hades.manager.InfoPanel
 *
 * GUI component to show name/author/version/description information
 * about HADES simobjects, designs, text files in Colibri
 *
 * 04.03.01 - converted to Swing GUI, FNH
 * 01.07.98 - original Colibri version by
 * @author    Ulrich Dallmann       (2dallman@informatik.uni-hamburg.de)
 * @author    Karola Kroenert       (2kroene@informatik.uni-hamburg.de
 */
package hades.manager;


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.EmptyBorder;


/**
 * InfoPanel:
 * GUI container to present component (=simobject, design) information.
 * <p>
 * This class builds the GUI needed to present component name, author,
 * version attributes, as well as a short text description (if available).
 *
 * layout:
   <pre>
   [ name field ...............]
   [ author: ] [ author field  ]
   [ version:] [ version field ]
   [ text area ... description ]
   </pre>
 */
public class  InfoPanel 
       extends  JPanel 
{
  private JTextField  nameTF;
  private JTextField  versionTF;
  private JTextField  authorTF;
  private JTextArea   infoTextArea;
  private JLabel      nameLabel;
  private JLabel      versionLabel;
  private JLabel      authorLabel;
  private JScrollPane infoTextScroller;

  
  public InfoPanel() 
  {
    super();

    nameTF       = new JTextField( "no name yet",    30 );
    authorTF     = new JTextField( "unknown author", 20 );
    versionTF    = new JTextField( "   1.001",       10 );
    nameLabel    = new JLabel( "name:", SwingConstants.RIGHT );
    authorLabel  = new JLabel( "author:", SwingConstants.RIGHT );
    versionLabel = new JLabel( "Version:", SwingConstants.RIGHT );

    infoTextArea = new JTextArea( "no description", 10, 80 );
    infoTextArea.setFont( new Font( "Courier", 0, 11 ));
    infoTextArea.setBorder( new EmptyBorder(10,10,10,10) );
    infoTextScroller = new JScrollPane( infoTextArea );
    //infoTextScroller.setBorder( new EmptyBorder(10,5,10,10) );
   
    JPanel labelPanel = new JPanel( new GridLayout(3,0,10,2) );
    labelPanel.add( nameLabel ); 
    labelPanel.add( authorLabel ); 
    labelPanel.add( versionLabel ); 
    labelPanel.setBorder( new EmptyBorder(10,10,10, 5) );

    JPanel tfPanel = new JPanel( new GridLayout(3,0,10,2) );
    tfPanel.add( nameTF ); 
    tfPanel.add( authorTF ); 
    tfPanel.add( versionTF ); 
    tfPanel.setBorder( new EmptyBorder(10,5,10,10) );

    JPanel fieldPanel = new JPanel();
    fieldPanel.setLayout( new BorderLayout() );
    fieldPanel.add( "West",  labelPanel );
    fieldPanel.add( "Center",tfPanel );

    this.setBackground( Color.yellow );
    this.setLayout( new BorderLayout() );
    this.add( "North",  fieldPanel );
    this.add( "Center", infoTextScroller );
  }



  public void setName( String name ) {
    nameTF.setText( name );
  }

  public void setAuthor( String name ) {
    authorTF.setText( name );
  }

  public void setVersion( String numerical ) {
    versionTF.setText( numerical );
  }

  public void setInfoText( String s ) {
    infoTextArea.setText( s );
    infoTextArea.setCaretPosition( 0 );
    //infoTextScroller.getViewport().setViewPosition( new Point(0,0) );
  }  


  public static void main( String argv[] ) {
 
    JFrame frame = new JFrame( "InfoPanel selftest..." );
    InfoPanel ip = new InfoPanel();
    ip.setName( "And2" );
    ip.setAuthor( "FNH" );
    ip.setVersion( "1.001" );
    ip.setInfoText( 
        "A simple AND gate with 2 inputs,\n"
      + "of type StdLogic1164 (UX01ZWLHD)\n"
      + "and gate-delay model.\n"
    );

    frame.getContentPane().add( ip );
    frame.setSize( 500, 200 );
    frame.show();
  }
}

