/* TreePopupMenu.java - hades.manager.TreePopupMenu
 *
 * manage the popup menus on the Colibri Tree control
 *
 * 01.03.01 - re-activated popup.show()
 * 27.02.01 - first separate version
 * 01.07.98 - original version in BrowserFrame by
 * @author    Ulrich Dallmann       (2dallman@informatik.uni-hamburg.de)
 * @author    Karola Kroenert       (2kroene@informatik.uni-hamburg.de
 */
package hades.manager;


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;

import hades.manager.tree.*;


public class TreePopupMenu 
       implements  ActionListener, MouseListener {

  JTree      tree;
  SortedTreeNode  popupNode;

  // root/local files/server/bookmark popup
  PopupMenu  popbook;
  MenuItem   popupdateb;
  MenuItem   poprem;
  MenuItem   popfolderb;

  // directory popup
  PopupMenu  popdir;
  MenuItem   popdeld;
  MenuItem   popupdated;
  MenuItem   popfolderd;

  // component popup
  PopupMenu  popcomp;
  MenuItem   popcreate;
  MenuItem   popopen;
  MenuItem   popdelc;
  MenuItem   popjar;
  MenuItem   pophtml;
  MenuItem   popprop;


  public TreePopupMenu( JTree tree ) {
    this.tree = tree;

    // Popup Menu for bookmarks/local
    popbook = new PopupMenu();
    popupdateb  = new MenuItem("Update");
    popupdateb.addActionListener(this);
    poprem  = new MenuItem("Remove");
    poprem.addActionListener(this);
    popfolderb = new MenuItem("Create subfolder");
    popfolderb.addActionListener(this);
    popbook.add(popupdateb);
    popbook.add(poprem);
    popbook.add(popfolderb);


    // Popup Menu for directories
    popdir = new PopupMenu();
    popdeld = new MenuItem("Delete");
    popdeld.addActionListener(this);     
    popupdated = new MenuItem("Update");
    popupdated.addActionListener(this);  
    popfolderd = new MenuItem("Create subfolder");
    popfolderd.addActionListener(this);
    popdir.add(popdeld);
    popdir.add(popupdated);
    popdir.add(popfolderd);


    // Popup Menu for components
    popcomp = new PopupMenu();
    popcreate = new MenuItem("Create");
    popcreate.addActionListener(this);
    popopen = new MenuItem("Open");
    popopen.addActionListener(this);     
    popdelc  = new MenuItem("Delete");
    popdelc.addActionListener(this);
    popjar  = new MenuItem("Create Jar-Archive");
    popjar.addActionListener(this);
    pophtml  = new MenuItem("Show HTML");
    pophtml.addActionListener(this);
    popprop = new MenuItem("Properties");
    popprop.addActionListener(this);     
    popcomp.add(popcreate);
    popcomp.add(popopen);
    popcomp.add(popdelc);
    popcomp.add(popjar);
    popcomp.add(pophtml);
    popcomp.add(popprop);

    tree.add(popbook);          
    tree.add(popdir);          
    tree.add(popcomp);          
    tree.addMouseListener( this );
  }


    

  /** 
   * handle action events 
   */
  public void actionPerformed( ActionEvent e ) 
  {
    String arg = (String) e.getActionCommand();
    // Popup-Menues
    // Popup-Menue for components
    // Properties

System.out.println( "-#- " + e );

/*
    if (e.getSource() == popprop)
    {
      TNode selected = (TNode)getSelected();
      // ComponentInfo
      if (selected!=null && selected.getUserObject() instanceof ComponentInfo) 
      {
        TNode tnode = selected.getTop();
        if (tnode.getUserObject() instanceof Bookmark)
          ((ComponentInfo)selected.getUserObject()).showDialog(this, true, false);
        else 
        {
          ComponentInfo clb = (ComponentInfo)selected.getUserObject();
          clb.showDialog(this, true, true);
          infoPanel.setInfo(clb);
          imageCanvas.setImage(clb.getImageObject());
        }
      }
    }
    // Delete
    else if (e.getSource() == popdelc)
    {
      TNode selected = (TNode)getSelected();
      if (selected!=null) 
      {
        Object[] options = {"Delete", "Cancel"};
        int erg = JOptionPane.showOptionDialog(this,"Are you sure, you want to delete the component and all its files ?", "Delete Component", JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE, null, options, options[0]);
        this.toFront();
        if (erg == 0) deleteNode(selected);
      }
    }
    // jar
    else if (e.getSource() == popjar)
    {
      TNode selected = (TNode)getSelected();
      if (selected!=null) 
      {
        String urlstring = getURLStringForNode(selected);
        ComponentInfo clb = (ComponentInfo)selected.getUserObject();
        SaveCommand sc = new SaveCommand();
        //sc.createJar(clb);
        //message("*******************************************");
        sc.createJar(urlstring);
      }
    }
    // html
    else if (e.getSource() == pophtml)
    {
      TNode selected = (TNode)getSelected();
      if (selected!=null) 
      {
        ComponentInfo clb = (ComponentInfo)selected.getUserObject();      
        TNode tnode = selected.getTop();
        String urlstring="";
        if (tnode.getUserObject() instanceof Bookmark)
          urlstring =  "http://"+clb.getServer()+"/"+clb.getRelativePath()+clb.getHighestVersion()+"/"+clb.getHTML();
        else if (tnode.getUserObject() instanceof String) 
        {
          if (tnode.getUserObject().toString().equals("local"))
          {
            urlstring =  "file:"+Options.getOptions().getLocalDir()+"/"+clb.getRelativePath()+clb.getHighestVersion()+"/"+clb.getHTML();
          }
          else
          {
            urlstring =  "file:"+Options.getOptions().getSysDir()+"/"+clb.getRelativePath()+clb.getHighestVersion()+"/"+clb.getHTML();
          }
        }
        message(urlstring);
        HtmlFrame help = new HtmlFrame(urlstring);
        help.setSize(700,550);
        help.setBackground(Color.white);
        help.show();
      }
    }
    // Open
    else if (e.getSource() == popopen)
    {
      TNode selected = (TNode)getSelected();
      if (selected!=null) 
      {
        opened(selected);
      }
    }
    // Popup-Menue directories
    // Update
    else if (e.getSource() == popupdated)
    {
      TNode selected = (TNode)getSelected();
      if (selected!=null) 
      {
        treePane.updateTree(selected);
      }
    }
    // Delete
    else if (e.getSource() == popdeld)
    {
      TNode selected = (TNode)getSelected();
      if (selected!=null) 
      {
        Object[] options = {"Delete", "Cancel"};
        int erg = JOptionPane.showOptionDialog
          (this,"Are you sure, you want to delete the directory and all its files ?",
          "Delete Component", JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE, 
          null, options, options[0]);
        this.toFront();
        if (erg == 0) deleteNode(selected);
      }
    }
    // Create subfolder
    else if (e.getSource() == popfolderd)
    {
      insertFolder();
    }
    
    // Popup-Menue bookmarks
    // Update
    else if (e.getSource() == popupdateb)
    {
      TNode selected = (TNode)getSelected();
      if (selected!=null) 
      {
        treePane.updateTree(selected);
      }
    }
    // Remove
    else if (e.getSource() == poprem)
    {
      TNode selected = (TNode)getSelected();
      // local and system can not be removed!
      // Bookmark
      if ((selected!=null)&&(selected.getUserObject() instanceof Bookmark))
      {
        Object[] options = {"Delete", "Cancel"};
        int erg = JOptionPane.showOptionDialog
          (this,"Are you sure, you want to remove the bookmark?", 
          "Delete Bookmark", JOptionPane.DEFAULT_OPTION, 
          JOptionPane.WARNING_MESSAGE, null, options, options[0]);
        this.toFront();
        if (erg == 0)
        {
          Bookmark bm = (Bookmark)selected.getUserObject();
          if (Options.getOptions().getBookmarks().removeElement(bm))
            removeFromTree(bm);
          Options.getOptions().saveOptions();
        }
      }
      // SearchResult
      else if ((selected!=null)&&(selected.getUserObject() instanceof String) 
        && selected.getUserObject().toString().startsWith("search"))
      {
        TNode parent = (TNode)selected.getParent();
        parent.remove(selected);
        ((DefaultTreeModel)treePane.getTree().getModel()).nodeStructureChanged(parent);
      }
    }
    // Create subfolder
    else if (e.getSource() == popfolderb)
    {
      insertFolder();
    }
*/
  }
  
  
    public void mouseClicked(MouseEvent e)
    {
      // component has been selected
      if(e.getClickCount() == 2)
      {
System.out.println( "-#- doubleClick!" );

        // find the TreeNode on which the user has clicked
        TreePath tmp = tree.getPathForLocation(e.getX(), e.getY());
        if (tmp == null) return;

        SortedTreeNode node = null;
        try {
          node = (SortedTreeNode) tmp.getLastPathComponent();
          node.handleDoubleClick( null );
        }
        catch( Throwable t ) {
System.out.println( "-F- internal: " + t );
        }
      }
   }

    public void mouseMoved( MouseEvent e ) {
      // empty
    }
    public void mouseDragged( MouseEvent e ) {
      // empty
    }
    public void mouseEntered( MouseEvent e ) {
      // empty
    }
    public void mouseExited( MouseEvent e ) {
      // empty
    }

    public void mouseReleased(MouseEvent e)
    {
      if (e.isPopupTrigger()) showPopup(e);
    }

    public void mousePressed(MouseEvent e)
    {
      if (e.isPopupTrigger()) showPopup(e);
    }


  private void showPopup(MouseEvent e)
  {
    // find the TreeNode on which the user has clicked
    TreePath tmp = tree.getPathForLocation(e.getX(), e.getY());
    if (tmp == null) return;

    SortedTreeNode node = null;
    try {
      node = (SortedTreeNode) tmp.getLastPathComponent();
      popupNode = node; // save reference to handle ActionEvents later

      if (node instanceof DirectoryTreeNode) {
         popdir.show((Component)e.getSource(), e.getX(), e.getY());  
      }
      else if (node instanceof SimObjectTreeNode) {
         popcomp.show((Component)e.getSource(), e.getX(), e.getY());  
      }
      else if (node instanceof DesignFileTreeNode) {
         popcomp.show((Component)e.getSource(), e.getX(), e.getY());  
      }
      else if (node instanceof ZipRootTreeNode) {
         popbook.show((Component)e.getSource(), e.getX(), e.getY());  
      }
    }
    catch( Exception ee ) {
       System.out.println( "-E- internal: " + ee );
       ee.printStackTrace();
    }

System.out.println( "-#- " + tmp );
System.out.println( "-S- " + tree.getSelectionPath() );

/*
      if (tmp != null)
      {
        TNode tn = (TNode)tmp.getLastPathComponent();
        if (tn != null)
        {
          // check if tn is part of a SearchResult
          boolean search = tn.getTop().getUserObject().toString().startsWith("search");
          // find out what type the node is and show the corresponding popup menu
          // system tree node
          if ((tn.getUserObject() instanceof String)&&(((String)tn.getUserObject()).equals("system")))
          {
            // disable remove for the 'system' tree
            poprem.setEnabled(false);
            popfolderb.setEnabled(false);
            popupdateb.setEnabled(!search);
            popbook.show((Component)e.getSource(), e.getX(), e.getY());  // pop up the menu.
          }
          // local tree node
          else if ((tn.getUserObject() instanceof String)&&(((String)tn.getUserObject()).equals("local")))
          {
            // disable remove for the 'local' tree
            poprem.setEnabled(false);
            popfolderb.setEnabled(!search);
            popupdateb.setEnabled(!search);
            popbook.show((Component)e.getSource(), e.getX(), e.getY());  // pop up the menu.
          }
          // search result
          else if ((tn.getUserObject() instanceof String)&&(((String)tn.getUserObject()).startsWith("search")))
          {
            // ensable remove for the 'search' tree
            poprem.setEnabled(true);
            popfolderb.setEnabled(false);
            popupdateb.setEnabled(false);
            popbook.show((Component)e.getSource(), e.getX(), e.getY());  // pop up the menu.
          }
          // bookmark
          else if (tn.getUserObject() instanceof Bookmark)
          {
            // enable remove for bookmarks
            poprem.setEnabled(!search);
            popfolderb.setEnabled(false);
            popupdateb.setEnabled(!search);
            popbook.show((Component)e.getSource(), e.getX(), e.getY());  // pop up the menu.
          }
          
          // directory
          else if (tn.getUserObject() instanceof DirObject)
          {
            popdeld.setEnabled(tn.getTop().getUserObject().toString().startsWith("local"));
            popupdated.setEnabled(!search);
            popfolderd.setEnabled(tn.getTop().getUserObject().toString().startsWith("local"));
            popdir.show((Component)e.getSource(), e.getX(), e.getY());  // pop up the menu.
          }
          // component
          else if (tn.getUserObject() instanceof ComponentInfo)
          {
            // enable "Delete" only on 'local' components
            Object obj = (tn.getTop()).getUserObject();
            ComponentInfo cinf = (ComponentInfo)tn.getUserObject();
            popdelc.setEnabled((obj instanceof String)&&((String)obj).equals("local"));
            popopen.setEnabled(cinf.showOpen());
            pophtml.setEnabled(cinf.getHTML()!=null);
            // show popup menu
            popcomp.show((Component)e.getSource(), e.getX(), e.getY()); // pop up the menu.
          }
        }
      }
*/
    }
  
}
