/* SimObjectTreeNode - hades.manager.tree.SimObjectTreeNode
 *
 * node for a SimObject (or subclass) in the Colibri tree
 *
 * 12.03.01 - use ComponentInfo/CLB file information, use lazy initialization
 * 04.03.01 - handleTreeSelection
 * 01.03.01 - first version
 */

package hades.manager.tree;

import javax.swing.ImageIcon;

import hades.manager.*;
import hades.gui.ColibriCreateCommand;
import hades.simulator.SimObject;
import hades.models.Design;



public class  SimObjectTreeNode
       extends  SortedTreeNode {


  ComponentInfo  clb = null;
  String        name = null;


  static ImageIcon _icon = loadIcon( "/hades/manager/icons/simobj.gif" );

  public ImageIcon getIcon() {
    return _icon;
  }


  /** 
   * load the ComponentInfo CLB file for this tree node.
   * The name of the CLB file is taken from the TreeManager full pathname.
   * For startup performance reasons, the CLB loading is delayed until this
   * tree node is first selected.
   */
  public void loadComponentInfo() {
msg( "-#- loading CLB file..." );
    try {
      clb = new ComponentInfo( TreeManager.getFullPathname(this) );
    }
    catch( Exception e ) {
msg( "-E- could not load ComponentInfo .clb file: " + e );
    }
  }


  /**
   * load and set the Icon for this SimObject,
   * load and display the name/author/version/description attributes
   * from a CLB and display in the Browser InfoPanel
   */
  public void handleTreeSelection( BrowserFrame browser ) {
    msg( "-#- SimObjTN.handleTreeSelection: " + toString() );

    if (clb == null) loadComponentInfo();

    String imagepath = clb.getImagepath();
    if (debug) msg( "-#- setting thumbnail from: " + imagepath ); 
    browser.setThumbnail(
      DesignManager.getDesignManager().getThumbnail( imagepath ));

    // FIXME: should load and parse ComponentInfo CLB
    InfoPanel ip = browser.getInfoPanel();
    ip.setName( toString() );
    ip.setAuthor( clb.getAuthor() );
    ip.setVersion( clb.getVersion() );
    ip.setInfoText( clb.getDescription() );
  }


  /**
   * handle a tree double click event on this node in the Colibri tree control.
   * For a SimObject, we try to create a new component in the current design
   * in the active editor.
   */
  public void handleDoubleClick( BrowserFrame browser ) {
    msg( "-#- SimObjectTN.handleDoubleClick: " + toString() );
    SimObject simObject = null;
    try {
      if (clb == null) loadComponentInfo();
      hades.gui.Editor editor;
      editor = hades.gui.Editor.getActiveEditor();

      // FIXME: should we really create a new editor?!
      //
      if (editor == null) editor  = new hades.gui.Editor();

if ("simobject".equals( clb.getType())) {
      simObject = 
        DesignManager.getDesignManager().
          getSimObject( clb.getStart() );
      simObject.setEditor( editor );
      if (!editor.isReady()) editor.doCancel();
      editor.setCommand( new ColibriCreateCommand( editor, simObject ));
}
else if ("design".equals( clb.getType())) {

      String initPath = clb.getInit();
      msg( "-#- Design: [init]= " + initPath );

      Design design = new Design();
      design.setEditor( editor );
      design.setVisible( true );
      design.initialize( initPath );
        
      if (!editor.isReady()) editor.doCancel();
      editor.setCommand( new ColibriCreateCommand( editor, design ));
    
}
else {
  msg( "-E- internal: unknown CLB type: " + clb.getType() );
  return;
}

    }
    catch( Exception ee ) {
      msg( "-E- SimObjectTN.handleDoubleClick: " + ee + " " );
      ee.printStackTrace();
    }
  } // handleDoubleClick



  /**
   * override setUserObject to also set our "name" for display in the tree:
   * we want to strip off any remaining directory pathname, and also to
   * strip off the ".clb" extension, if any. 
   * For example, called with the full pathname o="Xor2.clb", this 
   * method will set name="Xor2".
   */
  public void setUserObject( Object o ) {
    super.setUserObject( o );
    if (o instanceof String) {
      String s = (String) o;
      int  pos = Math.max( 0, s.lastIndexOf( '/' ));           // last '/' or 0
      int  end = s.lastIndexOf( '.' );                            // strip .clb
      if (end < 0) end = s.length();
      name = s.substring( pos, end );
    }
    else {
      name = o.toString();
    }
    // System.out.println( "-#- SOTN.sUO: " + o + " " + name );
  }


  public String toString() {
    return name;
  }

}
